"""Cleaning dirty input text."""
from typing import Iterable, Optional

import re

import requests
from bs4 import BeautifulSoup
from cleaning_utils import clear_stop_char, clear_stop_words, replace_arabic_char
from polyglot.detect import Detector


def fetch_page_text(url: str = "", webpage: str = "") -> str:
    """Getting html pages from urls & fetching needed elements of the page.

    Args:
        url: url of the target webpage.
        webpage: string version of the webpage.

    Returns:
        text of the webpage.
    """
    # If url is defined then make the request and fetch the page.
    if url != "":
        req = requests.get(url=url)
        soup = BeautifulSoup(req.content, "html.parser")

    # If not then make soup from given webpage.
    else:
        soup = BeautifulSoup(webpage, "html.parser")

    # Getting needed tags and stripped strings
    whole_page = ""
    for group in soup(["h1", "h2", "h3", "h4", "h5", "h6", "p"]):
        for text in group.stripped_strings:
            whole_page += text + " "

    whole_page = whole_page.strip()

    return whole_page


def cleaner(
    dirty_text: str,
    replace_stop: bool = False,
    stop_list: Optional[Iterable[str]] = None,
) -> str:
    """Main function for cleaning.

    Args:
        dirty_text: Input dirty text
        replace_stop: Whether to replace stop words or not
        stop_list: list of stop words

    Returns:
        Final text ready for integration in NLP algorithms.
    """
    # ------------------- HTML Stripper -------------------
    processed_text = re.sub('<[^<]+?>', '', dirty_text)
    processed_text = re.sub(
        '&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-fA-F]{1,6});', '', processed_text
    )
    # ------------------- Langugae detection -------------------
    detector = Detector(processed_text)
    lang = detector.language.code
    # ------------------- Linguistic phase -------------------
    if lang == "fa":
        processed_text = replace_arabic_char(processed_text)
    elif lang == "ar":
        processed_text = replace_arabic_char(processed_text, letter=False)

    processed_text = clear_stop_char(
        processed_text,
        replace_char=".",
    )

    if replace_stop:
        processed_text = clear_stop_words(
            text=processed_text, stop_list=stop_list, replace_char="."  # type: ignore
        )
        
    # ------------------- Trimmer phase -------------------
    processed_text = processed_text.replace("\t", " ").replace("\n", " ").strip()
    processed_text = processed_text.replace("\u200c", " ")  # Nim-fasele
    processed_text = re.sub(" +", " ", processed_text)  # space cleaner
    processed_text = processed_text.strip()

    return str(processed_text)



"""Main functions for counting phrases."""
from typing import Any, Optional

from hashlib import sha256
from pathlib import Path

import pandas as pd
from sklearn.feature_extraction.text import CountVectorizer

from phrase_counter.cleaner import cleaner, fetch_page_text

STOP_PATH = f"{Path(__file__).parent}/static/stop-words.txt"
with open(STOP_PATH, "r", encoding="utf-8") as stop_file:
    STOP_LIST = stop_file.readlines()

# Removing newlines
STOP_LIST = list(map(str.strip, STOP_LIST))


def ingest_doc(
    doc: str, doc_type: str = "TEXT", replace_stop: bool = False, tag_stop: bool = False
) -> Any:
    """Counting phrases in the document.

    Args:
        doc: Document to be processed.
        doc_type: URL, TEXT, or HTML.
        replace_stop: Whether to replace stop words or not
        tag_stop: Whether to change status of stop words or not

    Returns:
        Dataframe with counts for each word.

    Raises:
        Exception: If unkown value is given for doc_type.
    """
    # ---------------- Fetching the page text ----------------
    # Fetching a page from URL
    if doc_type == "URL":
        dirty_text = fetch_page_text(url=doc)

    # Processing HTML
    elif doc_type == "HTML":
        dirty_text = fetch_page_text(webpage=doc)

    # Raw text
    elif doc_type == "TEXT":
        dirty_text = doc

    else:
        raise Exception("Unknown value for doc_type argument.")

    cleaned_text = cleaner(dirty_text, replace_stop=replace_stop, stop_list=STOP_LIST)

    # ----------------- Initialization -----------------
    phrase_df = pd.DataFrame(columns=["bag", "count"])

    # ----------------- Splitting -----------------
    for text_part in cleaned_text.split("."):

        # ----------------- Counter Section -----------------
        # Initializing vector
        count_vector = CountVectorizer(ngram_range=(1, 5), encoding="utf-8")
        try:
            # Fit on text
            count_data = count_vector.fit_transform([text_part])
        except ValueError:
            continue

        # ----------------- Dataframe creation -----------------
        temp_df = pd.DataFrame(columns=["bag", "count"])
        temp_df["bag"] = count_vector.get_feature_names_out().tolist()
        temp_df["count"] = count_data.toarray()[0].tolist()

        # Concating results
        phrase_df = pd.concat([phrase_df, temp_df])

    # Creating phrase hash
    phrase_df["_key"] = phrase_df.apply(
        lambda row: sha256(row["bag"].encode()).hexdigest(), axis=1
    )

    # Aggregating for removing duplicate values (rows with same hash or phrase)
    phrase_df = phrase_df.groupby(["bag", "_key"]).agg({"count": "sum"}).reset_index()

    # Changing status to suggested stop for phrases that conatin stop words
    phrase_df["status"] = None
    if tag_stop:
        phrase_df["status"] = phrase_df.apply(
            lambda row: stop_word_detector(row["bag"]), axis=1
        )
    # Counting number of words in each bag
    phrase_df["length"] = phrase_df.apply(
        lambda row: len(str(row["bag"]).split()), axis=1
    )
    return phrase_df


def stop_word_detector(phrase: str) -> Optional[str]:
    """Find stop phrases based on existing list of stop words.

    Args:
        phrase: text of the phrase.

    Returns:
        status if stop words are in phrase.
    """
    # If there is any stop word in the phrase then it maybe a stop phrase
    if any(stop_word in phrase.split() for stop_word in STOP_LIST):
        return "suggested-stop"

    return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['phrase_counter']

package_data = \
{'': ['*'], 'phrase_counter': ['static/*']}

install_requires = \
['PyICU>=2.8.1,<3.0.0',
 'beautifulsoup4',
 'cleaning-utils>=0.1.5,<0.2.0',
 'pandas',
 'polyglot',
 'pycld2>=0.41,<0.42',
 'requests',
 'scikit-learn']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=4.5.0,<5.0.0']}

setup_kwargs = {
    'name': 'phrase-counter',
    'version': '0.1.4',
    'description': 'Counting stop words.',
    'long_description': '# Common Phrase Detection\n[![coverage report](assets/images/coverage.svg)](.logs/coverage.txt)\n[![static analysis](assets/images/mypy.svg)](.logs/mypy.txt)\n[![lint report](assets/images/pylint.svg)](.logs/pylint-log.txt)\n[![maintainability](assets/images/maintainability.svg)](.logs/maintainability.txt)\n[![complexity](assets/images/complexity.svg)](.logs/complexity.txt)\n[![Code style: black](assets/images/codestyle.svg)](https://github.com/psf/black)\n[![Pre-commit](assets/images/precommits.svg)](.pre-commit-config.yaml)\n[![license](assets/images/licence.svg)](https://github.com/rezashabrang/common-phrase-detection)\n\nThis is an API python library which is developed for detecting stop phrases.\n\n\n## Table of Contents\n\n- [Background](#background)\n- [Install](#install)\n- [API](#api)\n- [Maintainers](#maintainers)\n\n## Background\nNLP (Natural Language Processing) techniques is very helpful in various applications such as sentiment analysis, chatbots and other areas. For developing NLP models a need for a large & clean corpus for learning words relations is indisputable. One of the challanges in achieving a clean corpus is stop phrases. Stop phrases usually does not contain much information about the text and so must be identified and removed from the text.\n<br>\nThis is the aim of this repo to provide a structure for processing HTML pages (which are a valuable source of text for all languages) and finding a certain number of possible combinations of words and using human input for identifying stop phrases.\n\n## Install\n\n1. Make sure you have `docker`,`docker-compose` and `python 3.8` and above installed.\n\n2. create a `.env` file with desired values based on `.env.example` file.\n\n3. After cloning the project, go to the project directory and run below command.\n```bash\ndocker-compose -f docker-compose-dev.yml build\n```\n\n4. After the images are built successfully, run below command for starting the project.\n```bash\ndocker-compose -f docker-compose-dev.yml up -d\n```\n\n5. We need to create a database and collection in mongo in order to use the API. First run mongo bash.\n```\ndocker exec -it db bash\n```\n6. Authenticate in mongo container.\n```\nmongo -u ${MONGO_INITDB_ROOT_USERNAME} -p ${MONGO_INITDB_ROOT_PASSWORD} -- authenticationDatabase admin\n```\n7. Create the database and collection based on `MONGO_PHRASE_DB` and `MONGO_PHRASE_COL` names you provided in step `2`.\n```\nuse phrasedb;  # Database creation\ndb.createCollection("common_phrase");  # Collection creation\n```\n8. Now you\'re ready yo use the API section.\n\n## API\n\nThis API has three endpoints. <br>\n\n### Document Process\n\nHere you can pass a HTML text in request body to process it.\n\nThe process stages are:\n\n* Fetching all H1-H6 and p tags\n\n* Cleaning text\n* Finding bags (from 1 to 5 bags of word)\n* Counting the number of occurences in text\n* Integrating results in database\n(Updating count field of the phrase if already exists, otherwise inserting a\nnew record)\n\n### Status Updater\n\nUpdates statuses. <br>\n\nChanging the status of a phrase to either **stop** or **highlight**.\n\n### Data Fetcher\n\nFetching data from database based on the statuses.\nHere you can fetch phrases based on 4 different situation for statuses:\n\n* Stop phrases\n\n* Highlight phrases\n\n* Phrases that have status (either stop or highlight)\n\n* Phrases which statuses are not yet determined\n\n### API details\n\n* API Base URL\n```\n127.0.0.1:8000\n```\n* API Swagger UI\n```\n127.0.0.1:8000/docs\n```\nFor futher details and how to make request to each endpoint refer to the swagger of the API.\n\n## Maintainers\n[Maani Beygi](https://github.com/MaaniBeigy)<br>\n[Reza Shabrang](https://github.com/rezashabrang)\n',
    'author': 'aasaam',
    'author_email': 'rezashabrang.m@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rezashabrang/stop-word-counter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
